@@ -6,7 +6,7 @@ import sys
 import threading
 from abc import ABC, abstractmethod
 from collections import abc
-from dataclasses import dataclass, field, replace
+from dataclasses import dataclass, field
 from datetime import datetime
 from functools import wraps
 from getpass import getpass
@@ -20,10 +20,10 @@ from typing import (
     Dict,
     Iterable,
     List,
+    Mapping,
     NamedTuple,
     Optional,
     TextIO,
-    Tuple,
     Union,
     cast,
 )
@@ -32,7 +32,7 @@ from typing_extensions import Literal, Protocol, runtime_checkable
 
 from . import errors, themes
 from ._emoji_replace import _emoji_replace
-from ._log_render import LogRender, FormatTimeCallable
+from ._log_render import FormatTimeCallable, LogRender
 from .align import Align, AlignMethod
 from .color import ColorSystem
 from .control import Control
@@ -40,7 +40,8 @@ from .highlighter import NullHighlighter, ReprHighlighter
 from .markup import render as render_markup
 from .measure import Measurement, measure_renderables
 from .pager import Pager, SystemPager
-from .pretty import Pretty
+from .pretty import is_expandable, Pretty
+from .region import Region
 from .scope import render_scope
 from .screen import Screen
 from .segment import Segment
@@ -126,6 +127,8 @@ class ConsoleOptions:
     """Disable wrapping for text."""
     highlight: Optional[bool] = None
     """Highlight override for render_str."""
+    markup: Optional[bool] = None
+    """Enable markup when rendering strings."""
     height: Optional[int] = None
     """Height available, or None for no height limit."""
 
@@ -134,8 +137,19 @@ class ConsoleOptions:
         """Check if renderables should use ascii only."""
         return not self.encoding.startswith("utf")
 
+    def copy(self) -> "ConsoleOptions":
+        """Return a copy of the options.
+
+        Returns:
+            ConsoleOptions: a copy of self.
+        """
+        options = ConsoleOptions.__new__(ConsoleOptions)
+        options.__dict__ = self.__dict__.copy()
+        return options
+
     def update(
         self,
+        *,
         width: Union[int, NoChange] = NO_CHANGE,
         min_width: Union[int, NoChange] = NO_CHANGE,
         max_width: Union[int, NoChange] = NO_CHANGE,
@@ -143,12 +157,13 @@ class ConsoleOptions:
         overflow: Union[Optional[OverflowMethod], NoChange] = NO_CHANGE,
         no_wrap: Union[Optional[bool], NoChange] = NO_CHANGE,
         highlight: Union[Optional[bool], NoChange] = NO_CHANGE,
+        markup: Union[Optional[bool], NoChange] = NO_CHANGE,
         height: Union[Optional[int], NoChange] = NO_CHANGE,
     ) -> "ConsoleOptions":
         """Update values, return a copy."""
-        options = replace(self)
+        options = self.copy()
         if not isinstance(width, NoChange):
-            options.min_width = options.max_width = width
+            options.min_width = options.max_width = max(0, width)
         if not isinstance(min_width, NoChange):
             options.min_width = min_width
         if not isinstance(max_width, NoChange):
@@ -161,8 +176,10 @@ class ConsoleOptions:
             options.no_wrap = no_wrap
         if not isinstance(highlight, NoChange):
             options.highlight = highlight
+        if not isinstance(markup, NoChange):
+            options.markup = markup
         if not isinstance(height, NoChange):
-            options.height = height
+            options.height = None if height is None else max(0, height)
         return options
 
     def update_width(self, width: int) -> "ConsoleOptions":
@@ -172,9 +189,25 @@ class ConsoleOptions:
             width (int): New width (sets both min_width and max_width)
 
         Returns:
-            ~ConsoleOptions: New console options instance
+            ~ConsoleOptions: New console options instance.
         """
-        options = replace(self, min_width=width, max_width=width)
+        options = self.copy()
+        options.min_width = options.max_width = max(0, width)
+        return options
+
+    def update_dimensions(self, width: int, height: int) -> "ConsoleOptions":
+        """Update the width and height, and return a copy.
+
+        Args:
+            width (int): New width (sets both min_width and max_width).
+            height (int): New height.
+
+        Returns:
+            ~ConsoleOptions: New console options instance.
+        """
+        options = self.copy()
+        options.min_width = options.max_width = max(0, width)
+        options.height = height
         return options
 
 
@@ -210,6 +243,36 @@ class CaptureError(Exception):
     """An error in the Capture context manager."""
 
 
+class NewLine:
+    """A renderable to generate new line(s)"""
+
+    def __init__(self, count: int = 1) -> None:
+        self.count = count
+
+    def __rich_console__(
+        self, console: "Console", options: "ConsoleOptions"
+    ) -> Iterable[Segment]:
+        yield Segment("\n" * self.count)
+
+
+class ScreenUpdate:
+    """Render a list of lines at a given offset."""
+
+    def __init__(self, lines: List[List[Segment]], x: int, y: int) -> None:
+        self._lines = lines
+        self.x = x
+        self.y = y
+
+    def __rich_console__(
+        self, console: "Console", options: ConsoleOptions
+    ) -> RenderResult:
+        x = self.x
+        move_to = Control.move_to
+        for offset, line in enumerate(self._lines, self.y):
+            yield move_to(x, offset)
+            yield from line
+
+
 class Capture:
     """Context manager to capture the result of printing to the console.
     See :meth:`~rich.console.Console.capture` for how to use.
@@ -347,11 +410,13 @@ class RenderGroup:
             self._render = list(self._renderables)
         return self._render
 
-    def __rich_measure__(self, console: "Console", max_width: int) -> "Measurement":
+    def __rich_measure__(
+        self, console: "Console", options: "ConsoleOptions"
+    ) -> "Measurement":
         if self.fit:
-            return measure_renderables(console, self.renderables, max_width)
+            return measure_renderables(console, options, self.renderables)
         else:
-            return Measurement(max_width, max_width)
+            return Measurement(options.max_width, options.max_width)
 
     def __rich_console__(
         self, console: "Console", options: "ConsoleOptions"
@@ -385,8 +450,9 @@ def _is_jupyter() -> bool:  # pragma: no cover
         get_ipython  # type: ignore
     except NameError:
         return False
-    shell = get_ipython().__class__.__name__  # type: ignore
-    if shell == "ZMQInteractiveShell":
+    ipython = get_ipython()  # type: ignore
+    shell = ipython.__class__.__name__  # type: ignore
+    if "google.colab" in str(ipython.__class__) or shell == "ZMQInteractiveShell":
         return True  # Jupyter notebook or qtconsole
     elif shell == "TerminalInteractiveShell":
         return False  # Terminal running IPython
@@ -492,6 +558,8 @@ class Console:
         get_time (Callable[[], time], optional): Callable that gets the current time in seconds, default uses time.monotonic.
     """
 
+    _environ: Mapping[str, str] = os.environ
+
     def __init__(
         self,
         *,
@@ -523,10 +591,11 @@ class Console:
         safe_box: bool = True,
         get_datetime: Callable[[], datetime] = None,
         get_time: Callable[[], float] = None,
-        _environ: Dict[str, str] = None,
+        _environ: Mapping[str, str] = None,
     ):
         # Copy of os.environ allows us to replace it for testing
-        self._environ = os.environ if _environ is None else _environ
+        if _environ is not None:
+            self._environ = _environ
 
         self.is_jupyter = _is_jupyter() if force_jupyter is None else force_jupyter
         if self.is_jupyter:
@@ -596,6 +665,7 @@ class Console:
         self._record_buffer: List[Segment] = []
         self._render_hooks: List[RenderHook] = []
         self._live: Optional["Live"] = None
+        self._is_alt_screen = False
 
     def __repr__(self) -> str:
         return f"<console width={self.width} {str(self._color_system)}>"
@@ -863,7 +933,7 @@ class Console:
 
     def bell(self) -> None:
         """Play a 'bell' sound (if supported by the terminal)."""
-        self.control("\x07")
+        self.control(Control.bell())
 
     def capture(self) -> Capture:
         """A context manager to *capture* the result of print() or log() in a string,
@@ -913,9 +983,7 @@ class Console:
         """
 
         assert count >= 0, "count must be >= 0"
-        if count:
-            self._buffer.append(Segment("\n" * count))
-            self._check_buffer()
+        self.print(NewLine(count))
 
     def clear(self, home: bool = True) -> None:
         """Clear the screen.
@@ -923,7 +991,10 @@ class Console:
         Args:
             home (bool, optional): Also move the cursor to 'home' position. Defaults to True.
         """
-        self.control("\033[2J\033[H" if home else "\033[2J")
+        if home:
+            self.control(Control.clear(), Control.home())
+        else:
+            self.control(Control.clear())
 
     def status(
         self,
@@ -966,7 +1037,7 @@ class Console:
             show (bool, optional): Set visibility of the cursor.
         """
         if self.is_terminal and not self.legacy_windows:
-            self.control("\033[?25h" if show else "\033[?25l")
+            self.control(Control.show_cursor(show))
             return True
         return False
 
@@ -986,10 +1057,20 @@ class Console:
         """
         changed = False
         if self.is_terminal and not self.legacy_windows:
-            self.control("\033[?1049h\033[H" if enable else "\033[?1049l")
+            self.control(Control.alt_screen(enable))
             changed = True
+            self._is_alt_screen = enable
         return changed
 
+    @property
+    def is_alt_screen(self) -> bool:
+        """Check if the alt screen was enabled.
+
+        Returns:
+            bool: True if the alt screen was enabled, otherwise False.
+        """
+        return self._is_alt_screen
+
     def screen(
         self, hide_cursor: bool = True, style: StyleType = None
     ) -> "ScreenContext":
@@ -1026,15 +1107,15 @@ class Console:
             # No space to render anything. This prevents potential recursion errors.
             return
         render_iterable: RenderResult
-        if isinstance(renderable, RichCast):
-            renderable = renderable.__rich__()
-        if isinstance(renderable, ConsoleRenderable):
-            render_iterable = renderable.__rich_console__(self, _options)
+        if hasattr(renderable, "__rich__"):
+            renderable = renderable.__rich__()  # type: ignore
+        if hasattr(renderable, "__rich_console__"):
+            render_iterable = renderable.__rich_console__(self, _options)  # type: ignore
         elif isinstance(renderable, str):
-            yield from self.render(
-                self.render_str(renderable, highlight=_options.highlight), _options
+            text_renderable = self.render_str(
+                renderable, highlight=_options.highlight, markup=_options.markup
             )
-            return
+            render_iterable = text_renderable.__rich_console__(self, _options)  # type: ignore
         else:
             raise errors.NotRenderableError(
                 f"Unable to render {renderable!r}; "
@@ -1047,8 +1128,9 @@ class Console:
             raise errors.NotRenderableError(
                 f"object {render_iterable!r} is not renderable"
             )
+        _Segment = Segment
         for render_output in iter_render:
-            if isinstance(render_output, Segment):
+            if isinstance(render_output, _Segment):
                 yield render_output
             else:
                 yield from self.render(render_output, _options)
@@ -1072,27 +1154,37 @@ class Console:
             options (Optional[ConsoleOptions], optional): Console options, or None to use self.options. Default to ``None``.
             style (Style, optional): Optional style to apply to renderables. Defaults to ``None``.
             pad (bool, optional): Pad lines shorter than render width. Defaults to ``True``.
-            new_lines (bool, optional): Include "\n" characters at end of line.
+            new_lines (bool, optional): Include "\n" characters at end of lines.
 
         Returns:
             List[List[Segment]]: A list of lines, where a line is a list of Segment objects.
         """
         render_options = options or self.options
         _rendered = self.render(renderable, render_options)
-        if style is not None:
+        if style:
             _rendered = Segment.apply_style(_rendered, style)
         lines = list(
-            Segment.split_and_crop_lines(
-                _rendered,
-                render_options.max_width,
-                include_new_lines=new_lines,
-                pad=pad,
+            islice(
+                Segment.split_and_crop_lines(
+                    _rendered,
+                    render_options.max_width,
+                    include_new_lines=new_lines,
+                    pad=pad,
+                ),
+                None,
+                render_options.height,
             )
         )
         if render_options.height is not None:
-            lines = Segment.set_shape(
-                lines, render_options.max_width, render_options.height, style=style
-            )
+            extra_lines = render_options.height - len(lines)
+            if extra_lines > 0:
+                pad_line = [
+                    [Segment(" " * render_options.max_width, style), Segment("\n")]
+                    if new_lines
+                    else [Segment(" " * render_options.max_width, style)]
+                ]
+                lines.extend(pad_line * extra_lines)
+
         return lines
 
     def render_str(
@@ -1244,7 +1336,7 @@ class Console:
             elif isinstance(renderable, ConsoleRenderable):
                 check_text()
                 append(renderable)
-            elif isinstance(renderable, (abc.Mapping, abc.Sequence, abc.Set)):
+            elif is_expandable(renderable):
                 check_text()
                 append(Pretty(renderable, highlighter=_highlighter))
             else:
@@ -1279,14 +1371,15 @@ class Console:
         rule = Rule(title=title, characters=characters, style=style, align=align)
         self.print(rule)
 
-    def control(self, control_codes: Union["Control", str]) -> None:
+    def control(self, *control: Control) -> None:
         """Insert non-printing control codes.
 
         Args:
             control_codes (str): Control codes, such as those that may move the cursor.
         """
         if not self.is_dumb_terminal:
-            self._buffer.append(Segment.control(str(control_codes)))
+            for _control in control:
+                self._buffer.append(_control.segment)
             self._check_buffer()
 
     def out(
@@ -1357,8 +1450,7 @@ class Console:
                 Console default. Defaults to ``None``.
         """
         if not objects:
-            self.line()
-            return
+            objects = (NewLine(),)
 
         if soft_wrap is None:
             soft_wrap = self.soft_wrap
@@ -1384,9 +1476,10 @@ class Console:
             render_options = self.options.update(
                 justify=justify,
                 overflow=overflow,
-                width=min(width, self.width) if width else NO_CHANGE,
+                width=min(width, self.width) if width is not None else NO_CHANGE,
                 height=height,
                 no_wrap=no_wrap,
+                markup=markup,
             )
 
             new_segments: List[Segment] = []
@@ -1411,6 +1504,58 @@ class Console:
             else:
                 self._buffer.extend(new_segments)
 
+    def update_screen(
+        self,
+        renderable: RenderableType,
+        *,
+        region: Region = None,
+        options: ConsoleOptions = None,
+    ) -> None:
+        """Update the screen at a given offset.
+
+        Args:
+            renderable (RenderableType): A Rich renderable.
+            region (Region, optional): Region of screen to update, or None for entire screen. Defaults to None.
+            x (int, optional): x offset. Defaults to 0.
+            y (int, optional): y offset. Defaults to 0.
+
+        Raises:
+            errors.NoAltScreen: If the Console isn't in alt screen mode.
+
+        """
+        if not self.is_alt_screen:
+            raise errors.NoAltScreen("Alt screen must be enabled to call update_screen")
+        render_options = options or self.options
+        if region is None:
+            x = y = 0
+            render_options = render_options.update_dimensions(
+                render_options.max_width, render_options.height or self.height
+            )
+        else:
+            x, y, width, height = region
+            render_options = render_options.update_dimensions(width, height)
+
+        lines = self.render_lines(renderable, options=render_options)
+        self.update_screen_lines(lines, x, y)
+
+    def update_screen_lines(self, lines: List[List[Segment]], x: int = 0, y: int = 0):
+        """Update lines of the screen at a given offset.
+
+        Args:
+            lines (List[List[Segment]]): Rendered lines (as produced by :meth:`~rich.Console.render_lines`).
+            x (int, optional): x offset (column no). Defaults to 0.
+            y (int, optional): y offset (column no). Defaults to 0.
+
+        Raises:
+            errors.NoAltScreen: If the Console isn't in alt screen mode.
+        """
+        if not self.is_alt_screen:
+            raise errors.NoAltScreen("Alt screen must be enabled to call update_screen")
+        screen_update = ScreenUpdate(lines, x, y)
+        segments = self.render(screen_update)
+        self._buffer.extend(segments)
+        self._check_buffer()
+
     def print_exception(
         self,
         *,
@@ -1470,8 +1615,8 @@ class Console:
             _stack_offset (int, optional): Offset of caller from end of call stack. Defaults to 1.
         """
         if not objects:
-            self.line()
-            return
+            objects = (NewLine(),)
+
         with self:
             renderables = self._collect_renderables(
                 objects,
@@ -1566,7 +1711,7 @@ class Console:
         not_terminal = not self.is_terminal
         if self.no_color and color_system:
             buffer = Segment.remove_color(buffer)
-        for text, style, is_control in buffer:
+        for text, style, control in buffer:
             if style:
                 append(
                     style.render(
@@ -1575,7 +1720,7 @@ class Console:
                         legacy_windows=legacy_windows,
                     )
                 )
-            elif not (not_terminal and is_control):
+            elif not (not_terminal and control):
                 append(text)
 
         rendered = "".join(output)
@@ -1647,7 +1792,7 @@ class Console:
                 text = "".join(
                     segment.text
                     for segment in self._record_buffer
-                    if not segment.is_control
+                    if not segment.control
                 )
             if clear:
                 del self._record_buffer[:]
