@@ -3,7 +3,8 @@ import os
 import sys
 from array import array
 from collections import Counter, defaultdict, deque
-from dataclasses import dataclass
+from dataclasses import dataclass, fields, is_dataclass
+import inspect
 from itertools import islice
 from typing import (
     TYPE_CHECKING,
@@ -14,16 +15,19 @@ from typing import (
     List,
     Optional,
     Set,
+    Union,
     Tuple,
 )
 
 from rich.highlighter import ReprHighlighter
 
-from .abc import RichRenderable
 from . import get_console
+from ._loop import loop_last
 from ._pick import pick_bool
+from .abc import RichRenderable
 from .cells import cell_len
 from .highlighter import ReprHighlighter
+from .jupyter import JupyterRenderable
 from .measure import Measurement
 from .text import Text
 
@@ -61,7 +65,10 @@ def install(
     """
     from rich import get_console
 
+    from .console import ConsoleRenderable  # needed here to prevent circular import
+
     console = console or get_console()
+    assert console is not None
 
     def display_hook(value: Any) -> None:
         """Replacement sys.displayhook which prettifies objects with Rich."""
@@ -83,7 +90,44 @@ def install(
             )
             builtins._ = value  # type: ignore
 
-    sys.displayhook = display_hook
+    def ipy_display_hook(value: Any) -> None:  # pragma: no cover
+        assert console is not None
+        # always skip rich generated jupyter renderables or None values
+        if isinstance(value, JupyterRenderable) or value is None:
+            return
+        # on jupyter rich display, if using one of the special representations dont use rich
+        if console.is_jupyter and any(attr.startswith("_repr_") for attr in dir(value)):
+            return
+
+        # certain renderables should start on a new line
+        if isinstance(value, ConsoleRenderable):
+            console.line()
+
+        console.print(
+            value
+            if isinstance(value, RichRenderable)
+            else Pretty(
+                value,
+                overflow=overflow,
+                indent_guides=indent_guides,
+                max_length=max_length,
+                max_string=max_string,
+                expand_all=expand_all,
+                margin=12,
+            ),
+            crop=crop,
+        )
+
+    try:  # pragma: no cover
+        ip = get_ipython()  # type: ignore
+        from IPython.core.formatters import BaseFormatter
+
+        # replace plain text formatter with rich formatter
+        rich_formatter = BaseFormatter()
+        rich_formatter.for_type(object, func=ipy_display_hook)
+        ip.display_formatter.formatters["text/plain"] = rich_formatter
+    except Exception:
+        sys.displayhook = display_hook
 
 
 class Pretty:
@@ -101,6 +145,8 @@ class Pretty:
             Defaults to None.
         max_string (int, optional): Maximum length of string before truncating, or None to disable. Defaults to None.
         expand_all (bool, optional): Expand all containers. Defaults to False.
+        margin (int, optional): Subtrace a margin from width to force containers to expand earlier. Defaults to 0.
+        insert_line (bool, optional): Insert a new line if the output has multiple new lines. Defaults to False.
     """
 
     def __init__(
@@ -110,12 +156,14 @@ class Pretty:
         *,
         indent_size: int = 4,
         justify: "JustifyMethod" = None,
-        overflow: Optional["OverflowMethod"] = "crop",
+        overflow: Optional["OverflowMethod"] = None,
         no_wrap: Optional[bool] = False,
         indent_guides: bool = False,
         max_length: int = None,
         max_string: int = None,
         expand_all: bool = False,
+        margin: int = 0,
+        insert_line: bool = False,
     ) -> None:
         self._object = _object
         self.highlighter = highlighter or ReprHighlighter()
@@ -127,13 +175,15 @@ class Pretty:
         self.max_length = max_length
         self.max_string = max_string
         self.expand_all = expand_all
+        self.margin = margin
+        self.insert_line = insert_line
 
     def __rich_console__(
         self, console: "Console", options: "ConsoleOptions"
     ) -> "RenderResult":
         pretty_str = pretty_repr(
             self._object,
-            max_width=options.max_width,
+            max_width=options.max_width - self.margin,
             indent_size=self.indent_size,
             max_length=self.max_length,
             max_string=self.max_string,
@@ -146,22 +196,35 @@ class Pretty:
             no_wrap=pick_bool(self.no_wrap, options.no_wrap),
             style="pretty",
         )
-        pretty_text = self.highlighter(pretty_text)
+        pretty_text = (
+            self.highlighter(pretty_text)
+            if pretty_text
+            else Text(
+                f"{type(self._object)}.__repr__ returned empty string",
+                style="dim italic",
+            )
+        )
         if self.indent_guides and not options.ascii_only:
             pretty_text = pretty_text.with_indent_guides(
                 self.indent_size, style="repr.indent"
             )
+        if self.insert_line and "\n" in pretty_text:
+            yield ""
         yield pretty_text
 
-    def __rich_measure__(self, console: "Console", max_width: int) -> "Measurement":
+    def __rich_measure__(
+        self, console: "Console", options: "ConsoleOptions"
+    ) -> "Measurement":
         pretty_str = pretty_repr(
             self._object,
-            max_width=max_width,
+            max_width=options.max_width,
             indent_size=self.indent_size,
             max_length=self.max_length,
             max_string=self.max_string,
         )
-        text_width = max(cell_len(line) for line in pretty_str.splitlines())
+        text_width = (
+            max(cell_len(line) for line in pretty_str.splitlines()) if pretty_str else 0
+        )
         return Measurement(text_width, text_width)
 
 
@@ -187,12 +250,21 @@ _BRACES: Dict[type, Callable[[Any], Tuple[str, str, str]]] = {
     frozenset: lambda _object: ("frozenset({", "})", "frozenset()"),
     list: lambda _object: ("[", "]", "[]"),
     set: lambda _object: ("{", "}", "set()"),
-    tuple: lambda _object: ("(", ")", "tuple()"),
+    tuple: lambda _object: ("(", ")", "()"),
 }
 _CONTAINERS = tuple(_BRACES.keys())
 _MAPPING_CONTAINERS = (dict, os._Environ)
 
 
+def is_expandable(obj: Any) -> bool:
+    """Check if an object may be expanded by pretty print."""
+    return (
+        isinstance(obj, _CONTAINERS)
+        or (is_dataclass(obj) and not isinstance(obj, type))
+        or hasattr(obj, "__rich_repr__")
+    )
+
+
 @dataclass
 class Node:
     """A node in a repr tree. May be atomic or a container."""
@@ -205,6 +277,7 @@ class Node:
     last: bool = False
     is_tuple: bool = False
     children: Optional[List["Node"]] = None
+    key_separator = ": "
 
     @property
     def separator(self) -> str:
@@ -215,7 +288,7 @@ class Node:
         """Generate tokens for this node."""
         if self.key_repr:
             yield self.key_repr
-            yield ": "
+            yield self.key_separator
         if self.value_repr:
             yield self.value_repr
         elif self.children is not None:
@@ -312,7 +385,8 @@ class _Line:
         assert node.children
         if node.key_repr:
             yield _Line(
-                text=f"{node.key_repr}: {node.open_brace}", whitespace=whitespace
+                text=f"{node.key_repr}{node.key_separator}{node.open_brace}",
+                whitespace=whitespace,
             )
         else:
             yield _Line(text=node.open_brace, whitespace=whitespace)
@@ -374,17 +448,98 @@ def traverse(_object: Any, max_length: int = None, max_string: int = None) -> No
     def _traverse(obj: Any, root: bool = False) -> Node:
         """Walk the object depth first."""
         obj_type = type(obj)
-        if obj_type in _CONTAINERS:
-            obj_id = id(obj)
+        py_version = (sys.version_info.major, sys.version_info.minor)
+        children: List[Node]
 
+        def iter_rich_args(rich_args) -> Iterable[Union[Any, Tuple[str, Any]]]:
+            for arg in rich_args:
+                if isinstance(arg, tuple):
+                    if len(arg) == 3:
+                        key, child, default = arg
+                        if default == child:
+                            continue
+                        yield key, child
+                    elif len(arg) == 2:
+                        key, child = arg
+                        yield key, child
+                    elif len(arg) == 1:
+                        yield arg[0]
+                else:
+                    yield arg
+
+        if hasattr(obj, "__rich_repr__"):
+            args = list(iter_rich_args(obj.__rich_repr__()))
+
+            if args:
+                children = []
+                append = children.append
+                node = Node(
+                    open_brace=f"{obj.__class__.__name__}(",
+                    close_brace=")",
+                    children=children,
+                    last=root,
+                )
+                for last, arg in loop_last(args):
+                    if isinstance(arg, tuple):
+                        key, child = arg
+                        child_node = _traverse(child)
+                        child_node.last = last
+                        child_node.key_repr = key
+                        child_node.last = last
+                        child_node.key_separator = "="
+                        append(child_node)
+                    else:
+                        child_node = _traverse(arg)
+                        child_node.last = last
+                        append(child_node)
+            else:
+                node = Node(
+                    value_repr=f"{obj.__class__.__name__}()", children=[], last=root
+                )
+
+        elif (
+            is_dataclass(obj)
+            and not isinstance(obj, type)
+            and (
+                "__create_fn__" in obj.__repr__.__qualname__ or py_version == (3, 6)
+            )  # Check if __repr__ wasn't overriden
+        ):
+            obj_id = id(obj)
             if obj_id in visited_ids:
                 # Recursion detected
                 return Node(value_repr="...")
             push_visited(obj_id)
+
+            children = []
+            append = children.append
+            node = Node(
+                open_brace=f"{obj.__class__.__name__}(",
+                close_brace=")",
+                children=children,
+                last=root,
+            )
+
+            for last, field in loop_last(fields(obj)):
+                if field.repr:
+                    child_node = _traverse(getattr(obj, field.name))
+                    child_node.key_repr = field.name
+                    child_node.last = last
+                    child_node.key_separator = "="
+                    append(child_node)
+
+            pop_visited(obj_id)
+
+        elif obj_type in _CONTAINERS:
+            obj_id = id(obj)
+            if obj_id in visited_ids:
+                # Recursion detected
+                return Node(value_repr="...")
+            push_visited(obj_id)
+
             open_brace, close_brace, empty = _BRACES[obj_type](obj)
 
             if obj:
-                children: List[Node] = []
+                children = []
                 node = Node(
                     open_brace=open_brace,
                     close_brace=close_brace,
